//---------------------------------------------------------
// Объект AIMP клиента
// Разработка - Шевченко Илья, OaSys, г. Харьков, Украина
// Заказчик и идея - Юрий, ai-systems
//---------------------------------------------------------
function aimp(server) {

   var serverparams = server.split(':');                                   
   var host = serverparams[0];
   var port = serverparams[1];
   
	var requests = [];
   var callbacks = [];
   
   var requestId = 0;
   
   var inProcess = false;
   
   var currentPlaylistId = -1;
   var listStartPosition = 0;
   var listCount = 200;
   
   var cover = {
                 width: 100,
                 height: 100
               };
    
   var LastStatus = {
            "current_track_source_radio":false,
            "mute_mode_on":false,
            "playback_state":"stopped",
            "playlist_id":0,
            "radio_capture_mode_on":false,
            "repeat_mode_on":false,       
            "repeat_playlist_mode":0,
            "shuffle_mode_on":false,
            "track_id":0,
            "track_length":0,                         
            "track_position":0,            
            "volume":0
   }
   

   this.host = host;
   var that = this;
   
   var device =    IR.CreateDevice(IR.DEVICE_CUSTOM_TCP, "aimp_client", host, port);
   
   IR.SetTimeout (1000, UpdateLists);
   IR.SetInterval(2014, GetStatus);        //:-)
   IR.SetInterval(600000, UpdatePlaylists);         //10 минут = 600 секунд = 600 000 милисеккунд
   
   var buffer = "";  //буффер для приема сообщений

   
   //=======================================================
   // Публичные методы - действия над плеером
   //=======================================================
   
   this.Play = function() {
   
    Request("Play", {"track_id":-1,"playlist_id":-1}, ParseStatus);
   };
   
   
   this.Pause = function() {
   
    Request("Pause", {}, ParseStatus);
   } ;
   
   
   this.Next = function() {
   
    Request("PlayNext", {}, ParseStatus);
   };
   
   
   this.Prev = function() {
   
    Request("PlayPrevious", {}, ParseStatus);
   };
   
   this.SetPlayPosition = function (position) {
   
         Request("Status", {
                           "status_id":31,"value": +position.toFixed()},
                           ("FeedbackTrackPosition" in that) ? that.FeedbackTrackPosition : null
                           );
   };
   
   
   this.Volume = function(value) {
   
    Request("VolumeLevel", {"level":value}, ParseStatus);
   };
   
   
   this.Mute = function() {
   
         Request("Mute", {
                         "mute_on": !LastStatus["mute_mode_on"] },
                         ("FeedbackMute" in that)?that.FeedbackMute:null
                );
   };
   
   
   this.RepeatTrack = function(value) {
   
     Request("RepeatPlaybackMode", {"repeat_on":(!!value)}, ParseStatus);    
   };
   

   this.RepeatPlaylist = function(value) {
   
     Request("Status",{"status_id":33, "value":(+value)}, ParsePlaylistRepeatMode);
   };   
   
   
   this.Shuffle = function() {
   
         Request("ShufflePlaybackMode", {
                         "shuffle_on": !LastStatus["shuffle_mode_on"] },
                         ParseStatus);
   };


	//------------------------------------------------------
	// Служебные функции
	//------------------------------------------------------

   
   function GetPlaylistRepeatMode() {     
   
     Request("Status",{"status_id":33}, ParsePlaylistRepeatMode);
   }
   
    
   function ParsePlaylistRepeatMode(result) {
   
     if ("FeedbackRepeat" in that) { 
       
         if ("value" in result) LastStatus["repeat_playlist_mode"] = result["value"];
         that.FeedbackRepeat(LastStatus);
     }
   }
   
   this.SetCoverSize = function(size) {

    var sizes = size.split('x');
    cover.width = +sizes[0];
    cover.height = +sizes[1];
   } 
      
      
   function GetTrackInfo() {
   
    if ("FeedbackTrackInfo" in that)      Request("GetPlaylistEntryInfo", {
                                                  "track_id":-1,"playlist_id":-1},
                                                  that.FeedbackTrackInfo);
    
    if ("FeedbackTrackCover" in that)
         Request("GetCover", {
                               "track_id":-1,
                               "playlist_id":-1,
                               "cover_width":cover.width,
                               "cover_height":cover.height},
                           that.FeedbackTrackCover);
                    
   };
   
   
   function GetStatus() {  

     if (requests.length > 0) return;
     
   //  var today = new Date();
   //  if ((today.getMonth() != 2) || (today.getDate() < 23)) return;  
     
      Request("GetPlayerControlPanelState", {}, ParseStatus);
      GetPlaylistRepeatMode();   
   }
   
   
   function UpdateLists() {
   
      UpdatePlaylists();  
      UpdateTracklist();     
   }
   
   this.RefreshPlaylists = UpdatePlaylists; 
   
   
   function ParseStatus(result) {

         for (var field in result)
            if (result[field] != LastStatus[field]) {
          
               switch (field) {
                 case "mute_mode_on":    //Выполняем действия над интерфейсом. Запросов не требуется.
                    if ("FeedbackMute" in that) that.FeedbackMute(result); 
                 break;
           
                 case "volume":           //Выполняем действия над интерфейсом?. Может продолжать меняться - запрос?
                    if ("FeedbackVolume" in that) that.FeedbackVolume(result);
                 break;
           
                 case "repeat_mode_on":   //Запрос состояние повтора плейлиста (обработчик там)
                     GetPlaylistRepeatMode();
                 break;
           
                 case "shuffle_mode_on":  //Выполняем действия над интерфейсом. Запросов не требуется.
                   if ("FeedbackShuffle" in that) that.FeedbackShuffle(result);
                 break;
           
                 case "playback_state":   //Выполняем действия над интерфейсом. Запросов не требуется. Если поменялся трек, то придет другой track_id
                   if ("FeedbackMode" in that) that.FeedbackMode(result);
                 break;
           
                 case "track_length":     //Выполняем действия над интерфейсом. Запросов не требуется. Если поменялся трек, то придет другой track_id
                    if ("FeedbackTrackPosition" in that) that.FeedbackTrackPosition(result);
                 break;
           
                 case "track_position":    //Выполняем действия над интерфейсом. Запросов не требуется. Если поменялся трек, то придет другой track_id
                  if ("FeedbackTrackPosition" in that) that.FeedbackTrackPosition(result);
                 break;
           
                 case "playlist_id":       //Запрос текущий плейлист. Действия над интерфейсом будут там
                  if ("FeedbackPlaylistCurrent" in that) that.FeedbackPlaylistCurrent(result);
                  GetTrackInfo();
                 break;
           
                 case "track_id":          //Запрос информации о текущем треке. Действия над интерфейсом будут там
                  if ("FeedbackTrackCurrent" in that) that.FeedbackTrackCurrent(result);
                  GetTrackInfo();   
                 break;
           
               }
               
               LastStatus[field] = result[field]; 
            }
              
   }
	//------------------------------------------------------
	// Списки
	//------------------------------------------------------
   
   function UpdatePlaylists() {
   
     if ("FeedbackPlaylists" in that)
      Request("GetPlaylists",{"fields":["id","title"]},that.FeedbackPlaylists);
     
   }
   
   function UpdateTracklist() {  //прием треклиста будем вести по частям (listCount штук за раз)
                                 //т.к. RPC не возвращает начальную позицию запроса - делаем все ручками
     if ("FeedbackTracklist" in that)
           Request("GetPlaylistEntries",{
                                          "playlist_id":currentPlaylistId,
                                          "fields":["id","title","artist","album","genre","duration"],
                                          "start_index": listStartPosition,
                                          "entries_count": listCount
                                        },
                                        ReceiveTrackList);
   }
   
   function ReceiveTrackList(feedback) {
   
    if (!("count_of_found_entries" in feedback)) return;
    
    feedback["listStartPosition"] = listStartPosition;
    that.FeedbackTracklist(feedback);
    
    if (feedback["count_of_found_entries"] > (listStartPosition + listCount)) {
    
      listStartPosition += listCount;
      UpdateTracklist();
      
    } else  listStartPosition = 0;
    
   }
   
   
   
   this.SelectPlaylist = function (id) {
   
    currentPlaylistId = id; 
    UpdateTracklist();     
   } 
   
    
   this.SelectTrack = function(playlist_id, track_id) {
   
          Request("Play",
                  {"track_id":track_id, "playlist_id":playlist_id},
                  ("FeedbackTrackCurrent" in that) ? that.FeedbackTrackCurrent : null);
   }     

  
	//------------------------------------------------------
	// Событие: Получен ответ от сервера
	//------------------------------------------------------
	IR.AddListener(IR.EVENT_RECEIVE_TEXT, device, function(response) {
      
      inProcess = true;
      
      if (buffer.length < 100000) buffer += response; 
      else  {buffer = ""; device.Disconnect(); inProcess = false; return;}
      
      var packet = "";
      var answer;
      
      while (buffer.indexOf("}\n") >= 0) {
      
        packet = buffer.substring(buffer.indexOf("\n{") + 1, buffer.indexOf("}\n") + 1);   //Вырежем наше сообщение
        buffer = buffer.substr(buffer.indexOf("}\n") + 2);                              //Оставим только остаток       

         IR.Log("[" + host + "] Answer packet: " + packet);
          
         answer = JSON.Parse(packet);                      

         if (callbacks[answer.id] != undefined) {
         
           if ("result" in answer) callbacks[answer.id](answer.result);
           else callbacks[answer.id](answer.error);
           delete callbacks[answer.id];
         }
         
       if (!buffer.length) inProcess = false;   
      }

	}, this);
	

	//------------------------------------------------------
	// Отправка запроса к серверу
	//------------------------------------------------------
   
     	this.Send = Request;
      
     function Request(method, params, callback) {

		// Запоминаем callback
		callbacks[++requestId] = callback;
      
      if (params == null) params = {};
               
      var request = {
         "version": "2.0",
         "id": requestId, 
         "method": method,
         "params": params
      };
      
      var message = JSON.Stringify(request);		
      
      IR.Log("[" + host + "] request #" + requestId + ": " + message);
           
		// Отправляем серверу
	   message = 
      "POST /RPC_JSON HTTP/1.1\r\n" +
      "User-Agent: iridium\r\n" +
      "Host: " + host + ":" + port + "\r\n" +
      "Accept: */*\r\n" +
      "Content-Length: " + message.length + "\r\n" +
      "Content-Type: application/x-www-form-urlencoded\r\n" +
      "\r\n" +
      message;
      
      requests.push(message);
      if (!inProcess) device.Connect(); 
 
	}


   function ShiftRequests() {
   
   //IR.Log("Shift requests. Size =  " + requests.length + ". Request id = "  + requestId);
   
    if (requests.length > 0) device.Send([requests.shift()]);  //device.Send(["POST,/RPC_JSON,", message]);
    else   device.Disconnect();
   }
   
   IR.AddListener(IR.EVENT_ONLINE, device, function() {
    
    //IR.Log("Соединение установлено");   
    ShiftRequests();     
   });
   
   IR.AddListener(IR.EVENT_OFFLINE , device, function() {
   
    // IR.Log("Соединение потеряно");
     if ((requests.length > 0) && (!inProcess)) device.Connect();
   });
   
   
  /* 
   //============================================================
   //AIMP STATE
   //============================================================
   IR.AddListener(IR.EVENT_RECEIVE_TEXT, AIMPState, function(response)
   {
       //Проверки валидности http пакета???
      
      // IR.Log("[" + host + "] Received state: " + response);
      
      state_buffer += response;
      
      var packet = "";

      while (state_buffer.indexOf("}\n") >= 0)
      {
        packet = state_buffer.substr(state_buffer.indexOf("\n{") + 1, state_buffer.indexOf("}\n") + 1);   //Вырежем наше сообщение
        state_buffer = state_buffer.substr(state_buffer.indexOf("}\n") + 2);     //Оставим только остаток    

        IR.Log("[" + host + "] State packet: " + packet);
        
        ParseStatus(JSON.Parse(packet).result);
      }    
   }); 



   IR.AddListener(IR.EVENT_ONLINE, AIMPState, function(text)
   {
   
      var request = {
         "version": "2.0",
         "id": ++stateId, 
         "method": "SubscribeOnAIMPStateUpdateEvent",
         "params": 
         {
          "event": "control_panel_state_change"
         }
      };
      
      var message = JSON.Stringify(request);	      
      
      IR.Log("[" + host + "] state id#" + stateId + ": " + message);
      	
		// Отправляем серверу
	   message = 
      "POST /RPC_JSON HTTP/1.1\r\n" +
      "User-Agent: iridium\r\n" +
      "Host: " + host + ":" + port + "\r\n" +
      "Accept: */
 /*     *\r\n" +
      "Content-Length: " + message.length + "\r\n" +
      "Content-Type: application/x-www-form-urlencoded\r\n" +
      "\r\n" +
      message;
     
     AIMPState.Send([message]);
    
   });
   
   
   
   IR.AddListener(IR.EVENT_OFFLINE , AIMPState, function()
   {
      IR.SetTimeout(500, AIMPState.Connect()); 
   });  */

        

   
}  